@@ -1,7 +1,7 @@
 import funcy as fn
 from lazytree import LazyTree
 
-from dfa import DFA, ProductAlphabet
+from dfa import DFA
 
 
 def dfa2dict(dfa_):
@@ -40,33 +40,25 @@ def paths(dfa_, start, end=None, *, max_length=float('inf'), randomize=False):
     return (word for word, path in paths_ if end is None or path[-1] == end)
 
 
-def empty(inputs=None) -> DFA:
-    """Constructs DFA for the empty language."""
-    false = fn.constantly(False)
-    return DFA(start=False, label=false, transition=false, inputs=inputs)
+def find_word(lang: DFA):
+    """Returns a word in the language of DFA or None if language empty."""
+    accepted = (s for s in lang.states() if lang._label(s))
+    state = next(accepted, None)
+    if state is None:
+        return None
+    all_paths = paths(lang, lang.start, end=state)
+    return next(all_paths)
 
 
-def universal(inputs=None) -> DFA:
-    """Constructs DFA for the universal language."""
-    true = fn.constantly(True)
-    return DFA(start=True, label=true, transition=true, inputs=inputs)
+def find_equiv_counterexample(dfa_a, dfa_b):
+    """
+    Returns None if DFAs are equivalent; if not, returns a counterexample.
+    """
+    return find_word(dfa_a ^ dfa_b)
 
 
-def tee(left: DFA, right: DFA) -> DFA:
-    if left.inputs <= right.inputs:
-        inputs = left.inputs
-    elif right.inputs <= left.inputs:
-        inputs = right.inputs
-    else:
-        raise RuntimeError("Inputs need to be compatible")
-
-    def transition(s, c):
-        return left._transition(s[0], c), right._transition(s[1], c)
-
-    return DFA(
-        start=(left.start, right.start),
-        inputs=inputs,
-        transition=transition,
-        label=lambda s: (left._label(s[0]), right._label(s[1])),
-        outputs=ProductAlphabet(left.outputs, right.outputs),
-    )
+def find_subset_counterexample(smaller, bigger):
+    """
+    Returns None if smaller ⊆ bigger; if not, returns x ∈ smaller - bigger.
+    """
+    return find_word(~bigger & smaller)
